import * as React from 'react';

type WaveSurferCanvasDerivedComponent = HTMLAudioElement;

export interface WaveSurferCanvasProps extends React.HTMLProps<WaveSurferCanvasDerivedComponent> {}

export const WaveSurferCanvas = React.forwardRef<WaveSurferCanvasDerivedComponent, WaveSurferCanvasProps>(({
  className,
  children,
  controls,
  ...etcProps
}, forwardedRef) => {
  const [isPlaying, setIsPlaying] = React.useState(false);
  const defaultRef = React.useRef<HTMLAudioElement>(null);
  const ref = forwardedRef ?? defaultRef;
  const containerRef = React.useRef<HTMLDivElement>(null);
  const waveSurferRef = React.useRef<any>(null);

  const handleAction: React.FormEventHandler<HTMLFormElement> = (e) => {
    e.preventDefault();
    const formData = new FormData(e.currentTarget, (e.nativeEvent as unknown as { submitter: HTMLElement }).submitter);
    const action = formData.get('action');
    switch (action) {
    case 'togglePlayback':
      setIsPlaying((prev) => !prev);
      break;
    default:
      break;
    }
  };

  React.useEffect(() => {
    const { current: container } = containerRef;
    const media = typeof ref === 'object' ? ref?.current : null;
    const { current: waveSurferCurrent } = waveSurferRef;

    const load = async (ref: React.Ref<HTMLAudioElement>) => {
      if (!(typeof ref === 'object' && ref?.current)) {
        return;
      }

      if (!(typeof containerRef === 'object' && containerRef?.current)) {
        return;
      }
      const { default: WaveSurfer } = await import('wavesurfer.js');
      const waveSurferInstance = WaveSurfer.create({
        container: containerRef.current,
        barWidth: 2,
        barGap: 2,
        height: containerRef.current.clientHeight,
        fillParent: true,
        media,
      } as any);
      waveSurferInstance.load(ref.current.currentSrc);
      waveSurferInstance.on('ready', () => {
        if (!container) {
          return;
        }
        while (container.children.length > 1) {
          container.removeChild(container.children[0]);
        }
      });
      waveSurferRef.current = waveSurferInstance;
    };
    void load(ref);
    return () => {
      if (waveSurferCurrent) {
        (waveSurferCurrent as unknown as Record<string, Function>).destroy();
      }
    };
  }, [ref]);

  return (
    <div
      className="relative h-full w-full flex flex-col"
    >
      <audio
        {...etcProps}
        controls={controls}
        className={className}
        ref={ref}
      >
        {children}
      </audio>
      <div
        className="flex-auto relative"
      >
        <div className="absolute top-0 left-0 w-full h-full"
          ref={containerRef}
        />
      </div>
      {controls && (
        <form
          onSubmit={handleAction}
        >
          <button
            type="submit"
            name="action"
            value="togglePlayback"
          >
            {isPlaying ? '⏸' : '▶'}
          </button>
        </form>
      )}
    </div>
  );
});

WaveSurferCanvas.displayName = 'WavesurferCanvas';
